@@ -1,22 +1,27 @@
 #!/usr/bin/env python
 
-# code from http://effbot.org/zone/simple-top-down-parsing.htm
-# licence of original code was public domain
-# relicenced to AGPL v3 by Adrian Kalbarczyk and:
+# This file is part of ObjectPath released under AGPL v3 license.
+# Copyright (C) 2010-2014 Adrian Kalbarczyk
+
+# Code from http://effbot.org/zone/simple-top-down-parsing.htm was used in this file.
+# Licence of the code is public domain.
+# Relicenced to AGPL v3 by Adrian Kalbarczyk and:
 # - specialized to work with ObjectPath,
 # - optimized
+
 import sys
-if sys.version >= '3':
+
+if sys.version_info.major >= 3:
 	from io import StringIO
 else:
 	from cStringIO import StringIO
 
-from objectpath.core import *
+from objectpath.core import SELECTOR_OPS, NUM_TYPES
 
 symbol_table={}
-
-#TODO optimization ('-',1) -> -1
-#TODO optimization operators should be numbers
+token=nextToken=None
+# TODO optimization ('-',1) -> -1
+# TODO optimization operators should be numbers
 
 TRUE=["true","t"]
 FALSE=["false","f"]
@@ -30,7 +35,7 @@ class symbol_base(object):
 	def nud(self):
 		raise SyntaxError("Syntax error (%r)." % self.id)
 
-	def led(self, left):
+	def led(self):
 		raise SyntaxError("Unknown operator (%r)." % self.id)
 
 	def getTree(self):
@@ -49,11 +54,11 @@ class symbol_base(object):
 			fstLetter=self.value[0]
 			if fstLetter in ["'","\""]:
 				return self.value[1:-1]
-			#elif fstLetter.isdigit():
-			#	try:
-			#		return int(self.value)
-			#	except:
-			#		return float(self.value)
+			# elif fstLetter.isdigit():
+			# 	try:
+			# 		return int(self.value)
+			# 	except:
+			# 		return float(self.value)
 			else:
 				if self.value=="True":
 					return True
@@ -78,31 +83,38 @@ class symbol_base(object):
 				for j in i:
 					try:
 						t_append(j.getTree())
-					except:
+					except Exception:
 						t_append(j)
-				#TODO check if this is ever used?
-				if self.id == "[":
-					return t
+				if self.id in ("[",".",".."):
+					ret.append(t)
 				else:
 					ret.extend(t)
-				#ret_append(t)
-				#return (self.id,ret[1:])
+				# ret_append(t)
+				# return (self.id,ret[1:])
 			else:
-				if self.id=="-" and self.snd==None and type(self.fst.value) in [int, float]:
-					return -self.fst.value
-				if self.id=="+" and self.snd==None and type(self.fst.value) in [int, float]:
-					return self.fst.value
+				if type(self.fst.value) in NUM_TYPES and self.snd is None:
+					if self.id=="-":
+						return -self.fst.value
+					if self.id=="+":
+						return self.fst.value
 				ret_append(i.getTree())
+		if self.id == "{":
+			return {}
+		# if self.id == "[" and self.fst == []:
+		# 	return []
 		if self.id == "(":
-			#this will produce ("fn","fnName",arg1,arg2,...argN)
-			return tuple(["fn",ret[1][1]]+ret[2:])
+			# this will produce ("fn","fnName",arg1,arg2,...argN)
+			# try:
+				return tuple(["fn",ret[1][1]]+ret[2:])
+			# except:
+			# 	pass
 		return tuple(ret)
 
 	def __repr__(self):
 		if self.id == "(name)" or self.id == "(literal)":
-			return "(%s %s)" % (self.id[1:-1], self.value)
+			return "(%s:%s)" % (self.id[1:-1], self.value)
 		out=[self.id, self.fst, self.snd, self.third]
-		out=list(map(str, filter(None, out)))
+		# out=list(map(str, filter(None, out)))
 		return "(" + " ".join(out) + ")"
 
 def symbol(ID, bp=0):
@@ -147,7 +159,6 @@ def advance(ID=None):
 	if ID and token.id != ID:
 		raise SyntaxError("Expected %r, got %s"%(ID,token.id))
 	token=nextToken()
-	#print(token)
 
 def method(s):
 	# decorator
@@ -158,54 +169,72 @@ def method(s):
 
 infix_r("or", 30); infix_r("and", 40); prefix("not", 50)
 infix("in", 60); infix("not", 60) # not in
-infix("is", 60);
+infix("is", 60); infix("matches",60)
 infix("<", 60); infix("<=", 60)
 infix(">", 60); infix(">=", 60)
-#infix("	", 60); infix("!=", 60); infix("==", 60)
-#infix("|", 70); infix("^", 80); infix("&", 90)
-#infix("<<", 100); infix(">>", 100)
+# infix("	", 60); infix("!=", 60); infix("==", 60)
+# infix("&", 90)
+# infix("<<", 100); infix(">>", 100)
 infix("+", 110); infix("-", 110)
 infix("*", 120); infix("/", 120); infix("//", 120)
 infix("%", 120)
 prefix("-", 130); prefix("+", 130); #prefix("~", 130)
-#infix_r("**", 140)
-symbol(".", 150); symbol("[", 150); symbol("{", 150);symbol("(", 150)
-# additional behaviour
+# infix_r("**", 140)
+symbol(".", 150); symbol("[", 150); symbol("{", 150); symbol("(", 150)
+# additional behavior
 symbol("(name)").nud=lambda self: self
 symbol("(literal)").nud=lambda self: self
 symbol("(number)").nud=lambda self: self
 symbol("(end)")
 symbol(")")
 
+# REGEX
+infix("|", 0);
+infix("^", 0);
+infix("?", 0);
+infix("\\", 0);
+
 symbol("@")
 @method(symbol("@"))
-def nud(self):
+def nud(self): # pylint: disable=E0102
 	self.id="(current)"
 	return self
 
 symbol("!")
 @method(symbol("!"))
-def nud(self):
+def nud(self): # pylint: disable=E0102
 	self.id="(node)"
 	return self
 
-#RegEx
+# RegEx
 @method(symbol("/"))
-def nud(self):
+def nud(self): # pylint: disable=E0102
 	self.id="re"
-	self.fst=str(token.value)
-	advance()
+	regex=[]
+	if token.id != "/":
+		self_fst_append=regex.append
+		while 1:
+			if token.id == "/":
+				break
+			if token.id in ["(name)","(number)"]:
+				self_fst_append(str(token.value))
+			else:
+				self_fst_append(token.id)
+			advance()
+	self.fst="".join(regex).replace("\\","\\\\")
 	advance("/")
 	return self
 
 @method(symbol("("))
-def nud(self):
+def nud(self): # pylint: disable=E0102,W0613
 	expr=expression()
 	advance(")")
 	return expr
 
+symbol(",")
+
 @method(symbol("."))
-def led(self, left):
+def led(self, left): # pylint: disable=E0102
 	attr=False
 	if token.id == ".":
 		self.id=".."
@@ -213,7 +242,20 @@ def led(self, left):
 	if token.id == "@":
 		attr=True
 		advance()
-	if token.id not in ["(name)","*","(literal)" ]:
+	if token.id == "(":
+		advance()
+		self.fst=left
+		self.snd=[]
+		if token.id != ")":
+			self_snd_append=self.snd.append
+			while 1:
+				self_snd_append(expression())
+				if token.id != ",":
+					break
+				advance(",")
+		advance(")")
+		return self
+	if token.id not in ["(name)","*","(literal)","("]:
 		raise SyntaxError("Expected an attribute name.")
 	self.fst=left
 	if attr:
@@ -222,14 +264,13 @@ def led(self, left):
 	advance()
 	return self
 
-#handling namespaces; e.g $.a.b.c or $ss.a.b.c
-#default storage is the request namespace
+# handling namespaces; e.g $.a.b.c or $ss.a.b.c
+# default storage is the request namespace
 symbol("$")
 @method(symbol("$"))
-def nud(self):
-	global token
+def nud(self): # pylint: disable=E0102
+	global token # pylint: disable=W0602
 	self.id="(root)"
-	#print ("$.nud",token)
 	if token.id == ".":
 		self.fst="rs"
 	else:
@@ -240,7 +281,7 @@ def nud(self):
 symbol("]")
 
 @method(symbol("["))
-def led(self, left):
+def led(self, left): # pylint: disable=E0102
 	self.fst=left
 	self.snd=expression()
 	advance("]")
@@ -248,10 +289,10 @@ def led(self, left):
 
 symbol(",")
 
-#this is for built-in functions
+# this is for built-in functions
 @method(symbol("("))
-def led(self, left):
-	#self.id="fn"
+def led(self, left): # pylint: disable=E0102
+	# self.id="fn"
 	self.fst=left
 	self.snd=[]
 	if token.id != ")":
@@ -269,21 +310,21 @@ symbol("=")
 
 # constants
 
-def constant(id):
-	@method(symbol(id))
-	def nud(self):
+def constant(ID):
+	@method(symbol(ID))
+	def nud(self): # pylint: disable=W0612
 		self.id="(literal)"
-		self.value=id
+		self.value=ID
 		return self
 
 constant("None")
 constant("True")
 constant("False")
 
-#multitoken operators
+# multitoken operators
 
 @method(symbol("not"))
-def led(self, left):
+def led(self, left): # pylint: disable=E0102
 	if token.id != "in":
 		raise SyntaxError("Invalid syntax")
 	advance()
@@ -293,7 +334,7 @@ def led(self, left):
 	return self
 
 @method(symbol("is"))
-def led(self, left):
+def led(self, left): # pylint: disable=E0102
 	if token.id == "not":
 		advance()
 		self.id="is not"
@@ -304,7 +345,7 @@ def led(self, left):
 symbol("]")
 
 @method(symbol("["))
-def nud(self):
+def nud(self): # pylint: disable=E0102
 	self.fst=[]
 	if token.id != "]":
 		while 1:
@@ -320,7 +361,7 @@ def nud(self):
 symbol("}")
 
 @method(symbol("{"))
-def nud(self):
+def nud(self): # pylint: disable=E0102
 	self.fst={}
 	if token.id != "}":
 		while 1:
@@ -346,13 +387,14 @@ type_map={
 
 # python tokenizer
 def tokenize_python(program):
-	if sys.version < "3":
+	if sys.version_info.major < 3:
 		tokens=tokenizer.generate_tokens(StringIO(program).next)
 	else:
 		tokens=tokenizer.generate_tokens(StringIO(program).__next__)
 	for t in tokens:
+		# print type_map[t[0]], t[1]
 		try:
-			#change this to output python values in correct type
+			# change this to output python values in correct type
 			yield type_map[t[0]], t[1]
 		except KeyError:
 			if t[0] in [tokenizer.NL, tokenizer.COMMENT]:
@@ -368,17 +410,17 @@ def tokenize(program):
 		source=program
 	else:
 		source=tokenize_python(program)
-	for id, value in source:
-		if id=="(literal)":
-			symbol=symbol_table[id]
+	for ID, value in source:
+		if ID=="(literal)":
+			symbol=symbol_table[ID]
 			s=symbol()
 			s.value=value
-		elif id=="(number)":
-			symbol=symbol_table[id]
+		elif ID=="(number)":
+			symbol=symbol_table[ID]
 			s=symbol()
 			try:
 				s.value=int(value)
-			except:
+			except Exception:
 				s.value=float(value)
 		elif value == " ":
 			continue
@@ -387,13 +429,12 @@ def tokenize(program):
 			symbol=symbol_table.get(value)
 			if symbol:
 				s=symbol()
-			#elif id==" ":
-			elif id=="(name)":
-				symbol=symbol_table[id]
+			elif ID=="(name)":
+				symbol=symbol_table[ID]
 				s=symbol()
 				s.value=value
 			else:
-				raise SyntaxError("Unknown operator '%s', '%s'" % (id,value))
+				raise SyntaxError("Unknown operator '%s', '%s'" % (ID,value))
 		yield s
 
 # parser engine
@@ -401,31 +442,25 @@ def expression(rbp=0):
 	global token
 	t=token
 	token=nextToken()
-	#print(token)
 	left=t.nud()
-	#print("LEFT",left)
 	while rbp < token.lbp:
 		t=token
 		token=nextToken()
-		#print(token)
 		left=t.led(left)
 	return left
 
 def parse(expr, D=False):
-	if sys.version < "3" and type(expr) is unicode:
+	if sys.version_info.major < 3 and type(expr) is unicode:
 		expr=expr.encode("utf8")
 	if type(expr) is not str:
 		return expr
 	expr=expr.strip()
-	if not len(expr):
-		return Tree(True)
 	global token, nextToken
-	if sys.version >= "3":
+	if sys.version_info.major >= 3:
 		nextToken=tokenize(expr).__next__
 	else:
 		nextToken=tokenize(expr).next
 	token=nextToken()
-	#print(token)
 	r=expression().getTree()
 	if D:
 		print ("PARSE STAGE")
