@@ -32,9 +32,7 @@ class Debugger(object):
 		try:
 			self.doDebug=True
 			self.level=self.dbgMap["debug"]
-			#if app.dbg.has_key("cutAfter"):
-			#	self.CUT_AFTER=app.dbg["cutAfter"]
-			self.info("All strings will be cut to %s chatacters. You can change this behavior by setting cutAfter attribute of debuger in config file. 0 to switch it off."%self.CUT_AFTER)
+			self.info("All strings will be cut to %s chatacters."%self.CUT_AFTER)
 		except (KeyError, TypeError):
 			pass
 
@@ -75,11 +73,11 @@ class Debugger(object):
 			try:
 				if type(x) is unicode:
 					x=x.encode("utf8")
-			except:
+			except NameError:
 				pass
 			if self.CUT_AFTER and type(x) is dict:
 				s=[]
-				for i in x.iteritems():
+				for i in x.items():
 					s.append("'%s': %s"%(i[0],repr(i[1])[:self.CUT_AFTER]))
 					if len(s[-1])>self.CUT_AFTER:
 						s.append("...")
