@@ -1,21 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Asyncode Runtime - XML framework allowing developing internet
-# applications without using programming languages.
-# Copyright (C) 2008-2010  Adrian Kalbarczyk
-
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the version 3 of GNU General Public License as published by
-# the Free Software Foundation.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# This file is part of ObjectPath released under AGPL v3 license.
+# Copyright (C) 2008-2010 Adrian Kalbarczyk
 
 import datetime
 try:
@@ -23,21 +10,26 @@ try:
 	TIMEZONE_CACHE={
 		"UTC":pytz.utc
 	}
-except:
+except ImportError:
 	print("WARNING! pytz is not installed. Localized times are not supported.")
 
 HOURS_IN_DAY=24
 
+now=datetime.datetime.now
+
 def round9_10(n):
 	i=int(n)
 	if n-i>0.9:
 		return i+1
 	return i
 
-#TODO its 31 minuta, should be 31 minut - probably done
+# TODO its 31 minuta, should be 31 minut - probably done
+
+def age(date, reference=None, lang="en"):
+	if reference is None:
+		reference=now()
+	td=reference-date #TimeDelta
 
-def age(date,lang="en"):
-	td=now()-date
 	days=float(td.days)
 	langIsPL=lang=="pl"
 	if days:
@@ -47,23 +39,27 @@ def age(date,lang="en"):
 				return (years, years is 1 and "rok" or years<5 and "lata" or "lat")
 			else:
 				return (years, years is 1 and "year" or "years")
+
 		months=round9_10(days/30)
 		if months:
 			if langIsPL:
 				return (months, months is 1 and "miesiąc" or 1<months<5 and "miesiące" or "miesięcy")
 			else:
 				return (months, months is 1 and "month" or "months")
+
 		weeks=round9_10(days/7)
 		if weeks:
 			if langIsPL:
 				return (weeks, weeks is 1 and "tydzień" or weeks%10 in [0,1,5,6,7,8,9] and "tygodni" or "tygodnie")
 			else:
 				return (weeks, weeks is 1 and "week" or "weeks")
+
 		days=int(days)
 		if langIsPL:
 			return (days, days is 1 and "dzień" or "dni")
 		else:
 			return (days, days is 1 and "day" or "days")
+
 	seconds=float(td.seconds)
 	if seconds is not None:
 		hours=round9_10(seconds/3600)
@@ -72,20 +68,20 @@ def age(date,lang="en"):
 				return (hours, hours is 1 and "godzina" or 1<hours<5 and "godziny" or "godzin")
 			else:
 				return (hours, hours is 1 and "hour" or "hours")
+
 		minutes=round9_10(seconds/60)
 		if minutes:
 			if langIsPL:
 				return (minutes, minutes is 1 and "minuta" or 1<minutes<5 and "minuty" or "minut")
 			else:
 				return (minutes, minutes is 1 and "minute" or "minutes")
+
 		seconds=int(seconds)
 		if langIsPL:
 			return (seconds, seconds is 1 and "sekunda" or 1<seconds<5 and "sekundy" or "sekund")
 		else:
 			return (seconds, seconds is 1 and "second" or "seconds")
-	#return (0,"seconds")
-
-now=datetime.datetime.now
+	# return (0,"seconds")
 
 def date(d):
 	if d:
@@ -131,9 +127,9 @@ def addTimes(fst,snd):
 		if i>=60:
 			t2.append(i-60)
 			one=1
-		#elif i==60:
-		#	t2.append(0)
-		#	one=1
+		# elif i==60:
+		# 	t2.append(0)
+		# 	one=1
 		else:
 			t2.append(i)
 	hour=t[0]+one
@@ -173,11 +169,11 @@ def subTimes(fst,snd):
 def dateTime(arg):
 	"""
 	d may be:
-	 - datetime()
-	 - [y,m,d,h,m,ms]
-	 - [date(),time()]
-	 - [[y,m,d],[h,m,s,ms]]
-	 and permutations of above
+		- datetime()
+		- [y,m,d,h[,m[,ms]]]
+		- [date(),time()]
+		- [[y,m,d],[h,m,s,ms]]
+		and permutations of above
 	"""
 	l=len(arg)
 	if l is 1:
@@ -185,7 +181,7 @@ def dateTime(arg):
 		typed=type(dt)
 		if typed is datetime.datetime:
 			return dt
-		if typed in (tuple,list) and len(dt) in [5,7]:
+		if typed in (tuple,list) and len(dt) in [5,6,7]:
 			return datetime.datetime(*dt)
 	if l is 2:
 		date=time=None
@@ -204,10 +200,10 @@ def dateTime(arg):
 # dt - dateTime, tzName is e.g. 'Europe/Warsaw'
 def UTC2local(dt,tzName="UTC"):
 	try:
-		if TIMEZONE_CACHE.has_key(tzName):
+		if tzName in TIMEZONE_CACHE:
 			tz=TIMEZONE_CACHE[tzName]
 		else:
 			tz=TIMEZONE_CACHE[tzName]=pytz.timezone(tzName)
 		return TIMEZONE_CACHE["UTC"].localize(dt).astimezone(tz)
-	except:
+	except Exception:
 		return dt
